import pathlib
import unittest

from cleanghostimages import cleanghostimages


class TestCleanGhostImages(unittest.TestCase):

    ___input_files_dir = f"{ pathlib.Path(__file__).parent }/input_files"

    def test_absent_images(self):

        self.assertEqual(cleanghostimages.get_absent_images(["image1"], ["image1", "image2", "image3"]), ["image2", "image3"])
        self.assertEqual(cleanghostimages.get_absent_images(["image1", "image2"], ["image1", "image2"]), [])
        self.assertEqual(cleanghostimages.get_absent_images([], []), [])
        self.assertEqual(cleanghostimages.get_absent_images(["image1", "image2"], ["image1"]), [])

    def test_unused_images(self):
        self.assertEqual(cleanghostimages.get_unused_images(["image1"], ["image1"]), [])
        self.assertEqual(cleanghostimages.get_unused_images(["image1", "image2"], ["image1"]), ["image2"])
        self.assertEqual(cleanghostimages.get_unused_images(["image1"], ["image1", "image2"]), [])
        self.assertEqual(cleanghostimages.get_unused_images(["image1", "image2", "image3", "image5"], ["image1", "image3"]), ["image2", "image5"])

    def test_get_file_export_content(self):
        file_content = cleanghostimages.get_file_export_content(f"{ self.___input_files_dir }/test_can_read_file.txt")
        expected_file_content = """If this
is returned

  the test passes."""
        self.assertEqual(file_content, expected_file_content)
        self.assertRaises(FileNotFoundError, cleanghostimages.get_file_export_content, "iTEq7JTwyKx5ud8J75chRd3EzusPRV")

    def test_get_images_from_export_file(self):
        with open(f"{ self.___input_files_dir }/ghost_export_part.txt") as file:
            result = cleanghostimages.get_images_from_export_file(file.read())
            expected = [
                'content/images/2020/08/illustration12_o-1.png',
                'content/images/2022/12/vendors_compromised_service.png',
                'content/images/2020/08/illustration12_o-1.png',
                'content/images/2020/08/illustration12_o-1.png',
                'content/images/2022/12/vendors_compromised_service.png',
                'content/images/2022/12/vendors_compromised_service.png',
                'content/images/2020/08/Screenshot-from-2019-03-16-13-01-47.png',
                'content/images/2020/08/programming-583923_1280-1.jpg',
                'content/images/2020/11/123e.png',
                'content/images/2020/11/m554.jpg',
                'content/images/size/w600/2020/08/illustration12_o-1.png',
                'content/images/size/w1000/2020/08/illustration12_o-1.png',
                'content/images/size/w1600/2020/08/illustration12_o-1.png',
                'content/images/size/w600/2022/12/vendors_compromised_service.png'
            ]

            self.assertEqual(len(result), 14)
            self.assertEqual(result, expected)

    def test_get_sanitized_images_location_from_folder(self):
        ghost_install_dir = f"{ self.___input_files_dir }/test_get_images_from_folder"
        found_files = cleanghostimages.get_sanitized_images_location_from_folder(ghost_install_dir)
        expected_files = [
            'content/images/2022/01/dummy_file_3.png',
            'content/images/2022/12/dummy_file_3.jpeg',
            'content/images/size/w1000/2021/06/dummy_file_2.jpg',
            'content/images/size/w1000/2021/06/dummy_file_1.png'
        ]
        self.assertEqual(sorted(found_files), sorted(expected_files))

        # Test with an extra / to check that the content is still what is expected
        ghost_install_dir = f"{ self.___input_files_dir }//test_get_images_from_folder"
        found_files = cleanghostimages.get_sanitized_images_location_from_folder(ghost_install_dir)
        self.assertEqual(sorted(found_files), sorted(expected_files))

        self.assertRaises(FileNotFoundError, cleanghostimages.get_sanitized_images_location_from_folder, "/this/does/not/exist")

    def test_get_arguments_parser(self):
        parser = cleanghostimages.get_arguments_parser()

        # Test the required arguments
        self.assertRaises(SystemExit, parser.parse_args, "")
        self.assertRaises(SystemExit, parser.parse_args, "--json-export-file test".split(" "))
        self.assertRaises(SystemExit, parser.parse_args, "--ghost-dir test".split(" "))
        args = parser.parse_args("--ghost-dir test --json-export-file test2".split(" "))

        # Tests that the values are set where they should
        self.assertEqual(args.ghost_dir, "test")
        self.assertEqual(args.json_export_file, "test2")
        self.assertFalse(args.print_missing_images)
        self.assertFalse(args.statistics)
        self.assertFalse(args.print_unused_images)
        args = parser.parse_args("--json-export-file test1 --ghost-dir test2 --print-missing-images --statistics --print-unused-images".split(" "))
        self.assertTrue(args.print_missing_images)
        self.assertTrue(args.statistics)
        self.assertTrue(args.print_unused_images)



import os.path
import re
from pathlib import Path

import argparse
import numpy as np


def get_file_export_content(export_file_path: str) -> str:
    with open(export_file_path) as file:
        return file.read()


def get_images_from_export_file(file_content: str) -> list[str]:
    normal_images = re.findall('__GHOST_URL__/(content/images/[\d]{4}/[\d]{2}/[-\w.]+.[a-zA-Z]{2,4})', file_content)
    resized_images = re.findall('__GHOST_URL__/(content/images/size/w[a-zA-Z0-9]+/[\d]{4}/[\d]{2}/[-\w.]+.[a-zA-Z]{2,4})', file_content)
    return normal_images + resized_images


# Returns the relative location of all the images stored in the Ghost golder ("content/images/...")
def get_sanitized_images_location_from_folder(ghost_install_dir: str) -> list[str]:

    if not os.path.exists(ghost_install_dir):
        raise FileNotFoundError

    images_dir = f"{ghost_install_dir}/content/images/"
    posix_paths = [path for path in Path(images_dir).rglob("*") if not os.path.isdir(path)]
    # We want to remove the Ghost path so what is left is only the relative path
    # e.g. /opt/ghost/content/images/2022/01/img.png -> content/images/2022/01/img.png
    to_remove_from_path = ghost_install_dir.rstrip('/').replace('//', '/') + "/"
    return list(map(lambda posix_path: str(posix_path).replace(to_remove_from_path, ''), posix_paths))


def get_unused_images(images_in_dir: list[str], images_in_export: list[str]) -> list[str]:
    return np.setdiff1d(images_in_dir, images_in_export).tolist()


def get_absent_images(images_in_dir: list[str], images_in_export: list[str]) -> list[str]:
    return np.setdiff1d(images_in_export, images_in_dir).tolist()


def get_arguments_parser() -> argparse.ArgumentParser:
    parser = argparse.ArgumentParser(
        prog="cleanghostimages",
        description="A tool to find un-used images stored by your Ghost blog installation."
    )
    parser.add_argument("--json-export-file", type=str, required=True, help="The json export generated by Ghost")
    parser.add_argument("--ghost-dir", type=str, required=True, help="Your Ghost installation directory. Example /opt/ghost/")
    parser.add_argument("--print-missing-images", type=bool, action=argparse.BooleanOptionalAction)
    parser.add_argument("--statistics", type=bool, action=argparse.BooleanOptionalAction, help="Displays information about the number of images.")
    parser.add_argument("--print-unused-images", type=bool, action=argparse.BooleanOptionalAction)
    return parser


def execute():
    parser = get_arguments_parser()
    args = parser.parse_args()
    ghost_dir_full_path = os.path.abspath(args.ghost_dir)

    images_in_export = get_images_from_export_file(get_file_export_content(args.json_export_file))
    images_in_dir = get_sanitized_images_location_from_folder(ghost_dir_full_path)

    unused_files = get_unused_images(images_in_dir, images_in_export)
    absent_files = get_absent_images(images_in_dir, images_in_export)

    if args.statistics:
        print(f"Found a total of {len(images_in_export)} images in the export json.")
        print(f"Found a total of {len(images_in_dir)} images in the ghost content directory.")
        print(f"{len(unused_files)} images are not used, and {len(absent_files)} are missing.")

    if args.print_missing_images:
        print("The following images are missing:")
        for image in absent_files:
            print(image)

    if args.print_unused_images:
        print("The following images are not used:")
        for image in unused_files:
            print(image)
    return 0


if __name__ == "__main__":
    execute()



"""Install packages as defined in this file into the Python environment."""

from setuptools import setup, find_packages

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name="CleanGhostImages",
    author="Ixonae",
    author_email="contact@ixonae.com",
    description="A tool to find unused images in the Ghost blog content directory",
    version="0.2.1",
    packages=find_packages(where=".", exclude=["tests"]),
    long_description=long_description,
    long_description_content_type="text/markdown",
    python_requires=">=3.6",
    url="https://github.com/ixonae/CleanGhostImages",
    install_requires=[
        "setuptools>=45.0",
    ],
    entry_points={
        "console_scripts": [
            "cleanghostimages=cleanghostimages.__main__:main",
        ]
    },
    license="GNU GENERAL PUBLIC LICENSE v2"
)
